//
// Contains the main logic
//
var deliciousService = 
{
	urlPosts: "http://del.icio.us/api/posts/dates?",
	urlTags: "http://del.icio.us/api/tags/get?",
	usernameKey: 'delicious.username',
	passwordKey: 'delicious.password',
	usePopupKey: 'delicious.usePopup',
	isFirstTimeKey: 'delicious.isFirstTime',
	postUrlKey: 'delicious.postUrlKey',
	postPopupUrlKey: 'delicious.postPopupUrlKey',
	defaultPostUrl: 'http://del.icio.us/$username?url=$url&title=$title&extended=$extended',
	defaultPopupPostUrl: 'http://del.icio.us/$username?noui=yes&jump=close&url=$url&title=$title&extended=$extended',
	
	//
	// Get the URL of the del.icio.us post page
	//
	getPostUrl: function(username, url, title, extended)
	{
	    var postUrl = nsPreferences.copyUnicharPref(this.postUrlKey, "");   
	     
		// Use default if not specified
	    if(postUrl.length == 0)
	    {
    		nsPreferences.setUnicharPref(this.postUrlKey, this.defaultPostUrl);
    		postUrl = this.defaultPostUrl;
	    }
	    
	    postUrl = this.replaceVariables(postUrl, username, url, title, extended);
	    
	    return postUrl;
	},
	
	//
	// Get the URL of the del.icio.us popup post page
	//
	getPopupPostUrl: function(username, url, title, extended)
	{
	    var postUrl = nsPreferences.copyUnicharPref(this.postPopupUrlKey, "");    
		// Use default if not specified
	    if(postUrl.length == 0)
	    {
    		nsPreferences.setUnicharPref(this.postPopupUrlKey, this.defaultPopupPostUrl);
    		postUrl = this.defaultPopupPostUrl;
	    }

	    postUrl = this.replaceVariables(postUrl, username, url, title, extended);
	    
	    return postUrl;
	},
	
	//
	// Get the tags as an XML document
	//
	getTags: function()
	{
	    var username = this.getUsername();

	    if(username.length == 0)
	    {
		    return false;
	    }
	 	var password = this.getPassword();
	    return getURL(this.urlTags, username, password, true);
	},
	
	//
	// Get the posts as an XML document
	//
	getPosts: function()
	{
    	var username = this.getUsername();
	    var password = this.getPassword();
	    return getURL(this.urlPosts, username, password, true);
	},
	

	//
	// Returns the del.icio.us username or empty string if not specified.
	// But first it prompts the user for the username if it has not been specified
	//
	getUsername: function()
	{
	    var username = nsPreferences.copyUnicharPref(this.usernameKey, "");    
	    
	    if(username.length == 0)
	    {
	        alert("You have to specify a del.icio.us username");
	        this.showPreferences();
	        username = nsPreferences.copyUnicharPref(this.usernameKey, "");
	    }
	    return username;
	},
	
	//
	//
	//
	getPassword: function()
	{
	    var password = nsPreferences.copyUnicharPref(this.passwordKey, "");    
	    
	    if(password.length == 0)
	    {
	        alert("You have to specify a del.icio.us password");
	        this.showPreferences();
	        password = nsPreferences.copyUnicharPref(this.passwordKey, "");
	    }
	    return password;
	},
	
	//
	// 
	// 
	getUsePopup: function()
	{
	    var usePopup = nsPreferences.getBoolPref(this.usePopupKey, false);
	    return usePopup;
	},
	
	//
	//
	//
	setUsername: function(username)
	{
		nsPreferences.setUnicharPref(this.usernameKey, username);
	},
	
	//
	//
	//
	setPassword: function(password)
	{
    	nsPreferences.setUnicharPref(this.passwordKey, password);
	},
	
	//
	//
	//
	setUsePopup: function(enabled)
	{
    	nsPreferences.setBoolPref(this.usePopupKey, enabled);
	},
	
	//
	//
	//
	post: function(url, title, extended)
	{
	    // Get the username, default is empty
	    var username = this.getUsername();

	    if(username.length == 0)
	    {
		    return;
	    }
	    var usePopup = this.getUsePopup();
	    var postUrl;
        
	    // Check if we want should use the popup
	    if(usePopup)
	    {
	    	postUrl = this.getPopupPostUrl(username, url, title, extended);

	        open(postUrl, "delicious", "toolbar=no,width=700,height=250");
	    }
	    else
	    {
	    	postUrl = this.getPostUrl(username, url, title, extended);

	    	window._content.document.location = postUrl;
	    }
	},

	//
	//
	//
	showPreferences: function()
	{
		var preferencesDialog = window.openDialog('chrome://delicious/content/preferenceDialog.xul', 'del.icio.us', "modal,centerscreen,chrome,resizable=no");
	},
	
	
	//
	// Open the user's del.icio.us page
	//
	showMyDelicious: function()
	{
	    var username = this.getUsername();
	    
	    if(username.length == 0)
	    {
		    return;
	    }
	    var deliciousUrl = "http://del.icio.us/" + username;
	    window._content.document.location = deliciousUrl;
	},
	
	//
	// Open page showing the user's posts
	//
	showPosts: function()
	{
		// Checks if username & password is set
	    var username = this.getUsername();
	    var password = this.getPassword();

	    if(username.length == 0 || password.length == 0)
	    {
		    return;
	    }
	    var postsDialog = window.openDialog('chrome://delicious/content/postsDialog.xul', 'del.icio.us', "modal,centerscreen,chrome,resizable=no", window);
	},
	
	//
	// Show list of tags
	//
	showTags: function()
	{
		// Checks if username & password is set
	    var username = this.getUsername();
	    var password = this.getPassword();

	    if(username.length == 0 || password.length == 0)
	    {
		    return;
	    }
    	var tagsDialog = window.openDialog('chrome://delicious/content/tagsDialog.xul', 'del.icio.us', "modal,centerscreen,chrome,resizable=no", window);    
	},
	
	//
	//
	//
	showInbox: function()
	{
	    var username = this.getUsername();

	    if(username.length == 0)
	    {
		    return;
	    }
	    var deliciousUrl = "http://del.icio.us/inbox/" + username;
	    window._content.document.location = deliciousUrl;
	},
	
	//
	// Displays the selected posts for a date
	//
	showSelectedDate:function (date, target)
	{
	    var username = this.getUsername();
	   
	    var deliciousUrl;
	
	    deliciousUrl = "http://del.icio.us/" + username + "/" + date;
	    target.location.href = deliciousUrl;
	},
	
	//
	// Displays the selected tag
	//
	showSelectedTag:function(tag, allUsers, target)
	{
	    var username = this.getUsername();

    	var deliciousUrl;
    
	    if(allUsers)
	    {
	        deliciousUrl = "http://del.icio.us/tag/" + tag;
	    }
	    else
	    {
	        deliciousUrl = "http://del.icio.us/" + username + "/" + tag;
	    }
    
        target.location.href = deliciousUrl;
	},
	
	//
	// Replaces variables in the post URL
	//
	replaceVariables:function(postUrl, username, url, title, extended)
	{
		if(username.length == 0)
			username = "";
		if(url.length == 0)
			url = "";
		if(title.length == 0)
			title = "";
		if(extended.length == 0)
			extended = "";
	
	    // Replace the variables
	    postUrl = postUrl.replace("$username", username);
	    postUrl = postUrl.replace("$url", url);
	    postUrl = postUrl.replace("$title", title);
	    postUrl = postUrl.replace("$extended", extended);
	    return postUrl;
	},
    
    //
    // Finds the current page in del.icio.us
    //
    findURL:function(url)
    {
        var findUrl = "http://del.icio.us/url/" + hex_md5(url);
        window._content.document.location = findUrl;
    },
    
    //
    // Rename is not implemented in del.icio.us yet
    //
    renameTag:function(oldName, newName)
    {
		var url = "http://del.icio.us/api/tags/rename?old=";
		url += oldName;
		url += "&new=" + newName;
    	var username = this.getUsername();
	    var password = this.getPassword();
	    return getURL(url, username, password, true);
    }
};